/**
 * ASTASArg.java
 * 
 * Copyright (c) 2006 David Holroyd
 */

package uk.co.badgersinfoil.metaas.impl;

import org.antlr.runtime.RecognitionException;
import org.asdt.core.internal.antlr.AS3Parser;
import uk.co.badgersinfoil.metaas.SyntaxException;
import uk.co.badgersinfoil.metaas.dom.ASArg;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListTree;


public class ASTASArg implements ASArg {
	private LinkedListTree ast;

	public ASTASArg(LinkedListTree ast) {
		this.ast = ast;
	}

	public String getName() {
		return ast.getFirstChild().getText();
	}

	public String getType() {
		LinkedListTree type = ASTUtils.findChildByType(ast, AS3Parser.TYPE_SPEC);
		if (type == null) return null;
		return ASTUtils.identText(type.getFirstChild());
	}

	public void setType(String typeName) {
		LinkedListTree type = ASTUtils.findChildByType(ast, AS3Parser.TYPE_SPEC);
		if (typeName == null) {
			if (type != null) {
				ast.deleteChild(1);
			}
			return;
		}
		LinkedListTree newType = ASTUtils.newTypeSpec(typeName);
		if (type == null) {
			ast.addChildWithTokens(newType);
		} else {
			type.setChildWithTokens(0, newType.getFirstChild());
		}
	}

	public String toString() {
		String type = getType();
		if (type == null) {
			return getName();
		}
		return getName() + ":" + type;
	}

	public void setDefault(String value) {
		ASTIterator i = new ASTIterator(ast);
		LinkedListTree assign = i.search(AS3Parser.ASSIGN);
		if (value == null) {
			if (assign != null) {
				i.remove();
			}
			return;
		}
		AS3Parser parser = ASTUtils.parse("="+value);
		LinkedListTree def;
		try {
			def = ASTUtils.tree(parser.parameterDefault());
		} catch (RecognitionException e) {
			throw new SyntaxException(e.getMessage());
		}
		if (assign == null) {
			ast.addChildWithTokens(def);
		} else {
			assign.setChildWithTokens(0, def.getFirstChild());
		}
	}
}