/**
 * TagUtils.java
 * 
 * Copyright (c) 2006 David Holroyd
 */

package uk.co.badgersinfoil.metaas.impl;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;
import org.asdt.core.internal.antlr.AS3Parser;
import uk.co.badgersinfoil.metaas.SyntaxException;
import uk.co.badgersinfoil.metaas.dom.ASMetaTag;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListToken;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListTree;


public class TagUtils {

	public static List getAllMetaTags(LinkedListTree ast) {
		ASTIterator i = iterTags(ast);
		List result = new LinkedList();
		while (i.hasNext()) {
			result.add(toMetaTag(i.next()));
		}
		return Collections.unmodifiableList(result);
	}

	public static ASMetaTag getFirstMetaTag(LinkedListTree ast, String name) {
		ASTIterator i = iterTags(ast);
		while (i.hasNext()) {
			ASTASMetaTag tag = toMetaTag(i.next());
			if (tag.getName().equals(name)) {
				return tag;
			}
		}
		return null;
	}

	public static List getMetaTagWithName(LinkedListTree ast, String name) {
		ASTIterator i = iterTags(ast);
		List result = new LinkedList();
		while (i.hasNext()) {
			ASTASMetaTag tag = toMetaTag(i.next());
			if (tag.getName().equals(name)) {
				result.add(tag);
			}
		}
		return Collections.unmodifiableList(result);
	}

	public static ASMetaTag newMetaTag(LinkedListTree ast, String name) {
		// TODO: becase we have a hack that creates the ANNOTATIONS
		// with a NL token as its initial contents (bacacuse our
		// handling of AST nodes with no start/stop token is broken in
		// many situations), this code is inserting a NL at a different
		// point than expected:
		LinkedListTree tag = ASTFactory.newMetadataTag(name);
		LinkedListToken trailingnl = TokenBuilder.newNewline();
		tag.getStopToken().afterInsert(trailingnl);
		tag.setStopToken(trailingnl);
		String indent = ASTUtils.findIndent(ast);
		if (indent.length() > 0) {
			LinkedListToken startToken = tag.getStartToken();
			LinkedListToken indentTok = TokenBuilder.newWhiteSpace(indent);
			startToken.beforeInsert(indentTok);
			tag.setStartToken(indentTok);
		}
		findTags(ast).addChildWithTokens(tag);
		return toMetaTag(tag);
	}


	private static ASTIterator iterTags(LinkedListTree ast) {
		return new ASTIterator(findTags(ast));
	}

	private static LinkedListTree findTags(LinkedListTree ast) {
		return ASTUtils.findChildByType(ast, AS3Parser.ANNOTATIONS);
	}

	private static ASTASMetaTag toMetaTag(LinkedListTree tag) {
		if (tag.getType() != AS3Parser.ANNOTATION) {
			throw new SyntaxException("Expected ANNOTATION, got "+ASTUtils.tokenName(tag));
		}
		return new ASTASMetaTag(tag);
	}
}