/*
 * Copyright (c) David Holroyd
 */

package uk.co.badgersinfoil.metaas.impl.parser;

import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import org.antlr.runtime.ANTLRReaderStream;
import org.antlr.runtime.CharStream;
import org.antlr.runtime.RecognitionException;
import org.asdt.core.internal.antlr.AS3ParserLexer;
import uk.co.badgersinfoil.metaas.impl.ASTUtils;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListToken;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListTokenSource;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListTokenStream;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListTree;
import uk.co.badgersinfoil.metaas.impl.antlr.LinkedListTreeAdaptor;
import uk.co.badgersinfoil.metaas.impl.parser.e4x.E4X;
import uk.co.badgersinfoil.metaas.impl.parser.e4x.E4XLexer;


public class E4XHelper {
	private static final LinkedListTreeAdaptor TREE_ADAPTOR = new LinkedListTreeAdaptor();

	/**
	 * Creates a properly-configured parser object for the E4X grammar.
	 */
	public static E4X parserOn(Reader in) throws IOException {
		ANTLRReaderStream cs = new ANTLRReaderStream(in);
		E4XLexer lexer = new E4XLexer(cs);
		LinkedListTokenSource source = new LinkedListTokenSource(lexer);
		LinkedListTokenStream stream = new LinkedListTokenStream(source);
		E4X parser = new E4X(stream);
		parser.setTreeAdaptor(TREE_ADAPTOR);
		parser.setInput(lexer, cs);
		return parser;
	}

	public static LinkedListTree parseXMLLiteral(LinkedListToken startMarker, AS3ParserLexer lexer, CharStream cs, LinkedListTokenStream stream) throws RecognitionException {
		String tail = startMarker.getText()  // the '<'
		            + cs.substring(cs.index(), cs.size()-1);  // the rest
		E4X parser;
		try {
			parser = e4xParserOn(new StringReader(tail), stream);
		} catch (IOException e) {
			// TODO: better exception type?
			throw new RuntimeException(e);
		}
		LinkedListTree ast = ASTUtils.tree(parser.xmlPrimary());
		tail = parser.getInputTail();
		try {
			cs = new ANTLRReaderStream(new StringReader(tail));
		} catch (IOException e) {
			// TODO: better exception type?
			throw new RuntimeException(e);
		}
		lexer.setCharStream(cs);
		LinkedListTokenSource source = (LinkedListTokenSource)stream.getTokenSource();
		stream.setTokenSource(source);  // cause any remembered E4X state to be dropped
		source.setDelegate(lexer);
		return ast;
	}

	private static E4X e4xParserOn(Reader in, LinkedListTokenStream stream) throws IOException {
		ANTLRReaderStream cs = new ANTLRReaderStream(in);
		E4XLexer lexer = new E4XLexer(cs);
		LinkedListTokenSource source = (LinkedListTokenSource)stream.getTokenSource();
		stream.scrub(1); // erase the '<' and subsequent token that the AS3 parser got from this stream
		source.setDelegate(lexer);
		E4X parser = new E4X(stream);
		parser.setTreeAdaptor(new LinkedListTreeAdaptor());
		parser.setInput(lexer, cs);
		return parser;
	}

}